// TimeDlg.cpp : implementation file
//

#include "stdafx.h"
#include "malloc.h"
#include <afxsock.h>
#include "Time.h"
#include "TimeDlg.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#endif


#define SOCKET_CANNOT_INITIALIZE_STR " - Socket init failed"
#define SOCKET_CANNOT_CREATE_STR " - Socket not created"
#define SOCKET_CANNOT_CONNECT_STR " - Socket cannot connect"
#define SOCKET_CANNOT_RECEIVE_STR " - Socket receive problem"
#define INTERNAL_APP_ERROR "Internal application error"
#define SUCCESSFULLY_STR "Time set" 
#define CANNOT_SET_DATE_TIME_FORMAT " - Date & time cannot be set"
#define TIME_PROTOCOL 37
#define DATE_1970 2208988800


// CTimeDlg dialog
CTimeDlg::CTimeDlg(CWnd* pParent /*=NULL*/)
	: CDialog(CTimeDlg::IDD, pParent)
	, m_bThreadRunning(FALSE)
	, m_nThreadRet(-1)
	, m_nTimer(0)
{
	m_hIcon = AfxGetApp()->LoadIcon(IDR_MAINFRAME);
}

void CTimeDlg::DoDataExchange(CDataExchange* pDX)
{
	CDialog::DoDataExchange(pDX);
	DDX_Control(pDX, IDC_TIMESYNC, m_ctlTimeSync);
	DDX_Control(pDX, IDC_EDIT1, m_edtTimeServer);
}

BEGIN_MESSAGE_MAP(CTimeDlg, CDialog)
	ON_WM_PAINT()
	ON_WM_QUERYDRAGICON()
	//}}AFX_MSG_MAP
	ON_BN_CLICKED(IDOK, OnBnClickedOk)
	ON_BN_CLICKED(IDCANCEL, OnBnClickedCancel)
	ON_WM_TIMER()
END_MESSAGE_MAP()


// CTimeDlg message handlers

BOOL CTimeDlg::OnInitDialog()
{
	CDialog::OnInitDialog();

	m_edtTimeServer.SetWindowText("time.ien.it");
	m_ctlTimeSync.SetWindowText("");
	// Set the icon for this dialog.  The framework does this automatically
	//  when the application's main window is not a dialog
	SetIcon(m_hIcon, TRUE);			// Set big icon
	SetIcon(m_hIcon, FALSE);		// Set small icon

	// TODO: Add extra initialization here
	
	return TRUE;  // return TRUE  unless you set the focus to a control
}

// If you add a minimize button to your dialog, you will need the code below
//  to draw the icon.  For MFC applications using the document/view model,
//  this is automatically done for you by the framework.

void CTimeDlg::OnPaint() 
{
	if (IsIconic())
	{
		CPaintDC dc(this); // device context for painting

		SendMessage(WM_ICONERASEBKGND, reinterpret_cast<WPARAM>(dc.GetSafeHdc()), 0);

		// Center icon in client rectangle
		int cxIcon = GetSystemMetrics(SM_CXICON);
		int cyIcon = GetSystemMetrics(SM_CYICON);
		CRect rect;
		GetClientRect(&rect);
		int x = (rect.Width() - cxIcon + 1) / 2;
		int y = (rect.Height() - cyIcon + 1) / 2;

		// Draw the icon
		dc.DrawIcon(x, y, m_hIcon);
	}
	else		CDialog::OnPaint();
}

// The system calls this function to obtain the cursor to display while the user drags
//  the minimized window.
HCURSOR CTimeDlg::OnQueryDragIcon()
{
	return static_cast<HCURSOR>(m_hIcon);
}

void CTimeDlg::OnBnClickedOk()
{
	if (m_bThreadRunning) return;	// do not click again while querying for time

	m_nThreadRet = -1;
    m_ctlTimeSync.SetWindowText("");
	m_bThreadRunning = TRUE;
	m_nTimer = SetTimer(1, 1000,0);	// to check thread termination every sec
	m_TSyncThread = AfxBeginThread(SyncTime, this);	// start new thread
	m_ctlTimeSync.SetWindowText("Wait");
}

UINT CTimeDlg::SyncTime(LPVOID p)
{	
	CTimeDlg *me = (CTimeDlg*)p;
	return me->SyncTime();
}

int CTimeDlg::SyncTime(void)
{
	int nRetCode = 0;

	CSocket* pSocket; // Create socket
	WORD wVersionRequested;
	WSADATA wsaData;
	FILETIME FileTime, LocalFileTime;
	SYSTEMTIME SystemTime;
	UINT iErr=0;
	DWORD sBuffer=0;
	long lnTime=0;
	__int64 Time_Int64=0;

	pSocket= new CSocket();

	// Initialize sockets for Berkeley Protocol
	wVersionRequested = MAKEWORD( 1, 1 );
 
	iErr = WSAStartup( wVersionRequested, &wsaData );
	if ( iErr != 0 ) 
	{	
		nRetCode = 1000;	// should not fail
	}
 
	if ( LOBYTE( wsaData.wVersion ) != 1 ||	HIBYTE( wsaData.wVersion ) != 1 ) 
	{
		WSACleanup( );
		nRetCode = 1; 
	}

	pSocket->Create(); // Create socket
	if(0!=GetLastError())
	{
		delete pSocket;
		nRetCode = 1001;
	}
		
	CString TS;
	m_edtTimeServer.GetWindowText(TS);
	pSocket->Connect(TS, TIME_PROTOCOL); // Connect to time server
	if(0!=GetLastError())	nRetCode = 1002;


	pSocket->Receive((void*)&sBuffer, sizeof(sBuffer)); // Receive date & time
	if(0!=GetLastError())
	{
		pSocket->Close(); 
		delete pSocket; 
		nRetCode = 1003;
	}		

	if (nRetCode == 0)  // 
	{
		pSocket->Close();
		delete pSocket; 
	}
	else
	{
		m_nThreadRet = nRetCode;
		m_bThreadRunning = FALSE;
		return nRetCode;
	}


	sBuffer = htonl(sBuffer); // Reverse byte ordering

	// Set system time & date		
	lnTime = (long) (sBuffer - (unsigned long) DATE_1970);
	Time_Int64 = Int32x32To64(lnTime, 10000000) + 116444736000000000;
	FileTime.dwLowDateTime = (DWORD)Time_Int64;
	FileTime.dwHighDateTime = Time_Int64 >> 32;
	FileTimeToLocalFileTime(&FileTime, &LocalFileTime);
	FileTimeToSystemTime(&LocalFileTime, &SystemTime);
	BOOL bLocal = SetLocalTime( &SystemTime );

	if(0!=GetLastError())	nRetCode = 1005; // Time could not be set (maybe check user privileges)
		
	m_nThreadRet = nRetCode;
	m_bThreadRunning = FALSE;

	return m_nThreadRet;
}

void CTimeDlg::OnBnClickedCancel()
{
	// thread may be still running in the background
	// so kill it to be safe
	if (m_bThreadRunning)		delete m_TSyncThread;
	KillTimer(m_nTimer);
	OnCancel();
}

void CTimeDlg::OnTimer(UINT nIDEvent)
{
	CString str;
	if (!m_bThreadRunning)
	{
		CTime today;
		switch (m_nThreadRet)
		{
			case -1:	str = "Thread returned nothing!"; break;
			case 1000:	str = "Can't initialize socket"; break;
			case 1001:	str = "Can't create socket"; break;
			case 1002:	str = "Socket can't connect"; break;
			case 1003:	str = "Socket can't receive"; break;
			case 1005:	str = "Can't set system time"; break;
			default:
				{
					today = CTime::GetCurrentTime();
					str = today.Format("%H:%M:%S");
				};
		}
		m_ctlTimeSync.SetWindowText(str);	// display
	}
	else
	{
		m_ctlTimeSync.GetWindowText(str);
		str += ".";
		m_ctlTimeSync.SetWindowText(str);
	}
	CDialog::OnTimer(nIDEvent);
}
